/* ****************************************************************************
;                                                                             *
;    Project: AtomSD       				                              *
;    Date: 7/10/2011                               	                          *
;    File Version: 1.0                                                        *
;                                                                             *
;    Author:   Jason Lopez                                                    *
;    Company:  AtomSoftTech                                                   *
;                                                                             *
;******************************************************************************/

#include "main.h"

#pragma config WDT = OFF, LVP = OFF, OSC = HSPLL, XINST = OFF

FATFS fs;
FIL fsrc;
char isMounted = 0;

//Commands for UART
unsigned char CMD[8][5] = {"LIST","READ","WRIT","APND","LIFI","LIDI","FIFI","FIDI"};
char Buffer[128];
char ThePath[30];
char TheName[30];
char* endP;
char* endN;
char* dataP;
char NameLen;
unsigned char DataLen;

/************************************
Main Function
*************************************/
void main(void){
    UINT br;           // File read count
    FRESULT res;       // Petit FatFs function common result code

	TRISA = 0x00;
	TRISB = 0x00;
	TRISC = 0x00;
	TRISD = 0x00;
	
	ADCON1 = 0x0F;
	
	Init_UART();

	SD_CD_DIR = 1;
	CommandComplete = 0;

	while(1)
	{
      switch(isMounted)
        {
            case 0:
                if(SD_CD)
                    isMounted = 1;
                break;
            case 1:
                if(SD_CD)
                {			
					res = MountSD();
					if(res)
					{
						isMounted = 0;
					}
					else
					{
						isMounted++;
					}
                }
                else
                {
                  isMounted=0;
                }
                break;

            case 2:
                if(SD_CD == 0)
                    isMounted = 0;
                if(CommandComplete==1)
				{
                  RunCommand();
				  CommandComplete = 0;	//Command handled, free it
				  memset(RX_CMD,0,4);
				}
                break;
        }
    }
}

/************************************
Commands
*************************************/

void RunCommand(void)
{
	FRESULT res;
	UINT temp=0;
	UINT SeekPosition = 0;
	char temp2 = 0;
  	UINT br, bw;         /* File read/write count */
	char Bytes2Read = 0;
	char Bytes2Write = 0;

	char RWC[2]={0,0};

	if(strncmp(CMD[LIST],RX_CMD,4) == 0)	//Send out directory listing
	{
		res = scan_files(RX_ARG,ALL_T);
		return;
	}

	if(strncmp(CMD[LIST_FILES],RX_CMD,4) == 0)	//Send out directory listing
	{
		res = scan_files(RX_ARG,FILE_T);
		return;
	}
	if(strncmp(CMD[LIST_DIRS],RX_CMD,4) == 0)	//Send out directory listing
	{
		res = scan_files(RX_ARG,DIR_T);
		return;
	}

	if(strncmp(CMD[READ],RX_CMD,4) == 0)	//Read data from a file
	{
		SeekPosition = RX_ARG[3] + (RX_ARG[2] << 8)+ (RX_ARG[1] << 16)+ (RX_ARG[0] << 24); 
		Bytes2Read = RX_ARG[5];
		res = f_open(&fsrc,(RX_ARG+8),FA_READ);

		memset(Buffer,0,128); 

		res = f_lseek(&fsrc,SeekPosition);
      	res = f_read(&fsrc,Buffer, Bytes2Read, &br);    /* Read a chunk of src file */

		RWC[0] = br;		//Get how many actual bytes were read.
		UART_TX(RWC[0]);	//inform the user how many bytes to expect after this
      	UART_BUFF(Buffer,br);   //Send the buffer.

		f_close(&fsrc);
		return;
	}
	if(strncmp(CMD[FIND_FILE],RX_CMD,4) == 0)	//Send out directory listing
	{
		res = find_it(RX_ARG,FILE_T);
		UART_TX(res);
		return;
	}
	if(strncmp(CMD[FIND_DIR],RX_CMD,4) == 0)	//Send out directory listing
	{
		res = find_it(RX_ARG,DIR_T);
		UART_TX(res);
		return;
	}

	if(strncmp(CMD[WRITE],RX_CMD,4) == 0)	//Write data to a file
	{
		memset(ThePath,0,30);
		memset(TheName,0,30);

		DataLen = RX_ARG[0];
		dataP = RX_ARG + 2;

		//endP = strrchr(dataP,'/')+1;
		//strncpy(ThePath,dataP,(endP - dataP));

		endN = strrchr(dataP,CR);
		//strncpy(TheName,endP,(endN - dataP));

		strncpy(ThePath,dataP,(endN - dataP));

		//strcpy(TheName,endN);
		memset(Buffer,0,128); 
		strncpy(Buffer,(endN+1),DataLen);

		res = f_open(&fsrc,ThePath,FA_CREATE_ALWAYS | FA_WRITE);
        res = f_write(&fsrc, Buffer, DataLen, &bw); 
		
		f_close(&fsrc);

		if(res)
			UART_TX(0);
		else
			UART_TX(bw);

		return;
	}
	if(strncmp(CMD[APPEND],RX_CMD,4) == 0)	//Append data to a file
	{
		memset(ThePath,0,30);
		memset(TheName,0,30);

		DataLen = RX_ARG[0];
		dataP = RX_ARG + 2;

		//endP = strrchr(dataP,'/')+1;
		//strncpy(ThePath,dataP,(endP - dataP));

		endN = strrchr(dataP,CR);
		//strncpy(TheName,endP,(endN - dataP));

		strncpy(ThePath,dataP,(endN - dataP));

		//strcpy(TheName,endN);
		memset(Buffer,0,128); 
		strncpy(Buffer,(endN+1),DataLen);

		res = f_open(&fsrc,ThePath,FA_CREATE_ALWAYS | FA_WRITE);
		SeekPosition = fsrc.fsize;
		res = f_lseek(&fsrc,SeekPosition);
        res = f_write(&fsrc, Buffer, DataLen, &bw); 
		
		f_close(&fsrc);

		if(res)
			UART_TX(0);
		else
			UART_TX(bw);

		return;
	}
}
